/*
rbf_nn.cc

This C++ file defines member functions for the class RBF_NN_MODULE
It was generated with rcsdesign
with template version 1.1

MODIFICATIONS:
Wed Dec 20 17:11:24 EST 2000	Created with RCS-Design tool.

*/

// Include Files
#include "rcs.hh" 	// Common RCS definitions
#include "nml_mod.hh" 	// NML_MODULE definitions
#include "rbf_nn.hh" 	// RBF_NN_MODULE definitions
// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.


#include "rbf_nnn.hh" 	// NML Commands and Status definitions for rbf_nn
// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


// Constructor
RBF_NN_MODULE::RBF_NN_MODULE(int _is_base_class)
{

	// Initialize the NML channels if this module is not being used as the base class for another module.
	if(!_is_base_class)
	{
		INITIALIZE_NML();
	}
	// Add additional code to initialize the module here.
        num_of_inputs = NUM_OF_INPUTS;
        num_of_outputs = NUM_OF_OUTPUTS;
        num_of_neurons = NUM_OF_NEURONS;

	rbf = new RBF(num_of_inputs, num_of_outputs, num_of_neurons);
	
	if(rbf->rbf_err != RBF_NO_ERROR)
	  {
	    logError("Error in creating RBF_NN_MODULE object!!!");
	    status = RCS_ERROR; // let my supervisor know I have a problem.
	  }


    if ((temp = new double*[num_of_neurons]) == NULL)
      {
        logError("Error in creating RBF_NN_MODULE object!!!");
        status = RCS_ERROR; // let my supervisor know I have a problem.
      }
    for (int ii = 0; ii < num_of_neurons; ii++)
      {
        if ((temp[ii] = new double[num_of_inputs]) == NULL)
          {
            logError("Error in creating KALMAN_FILTER_MODULE object!!!");
            status = RCS_ERROR; // let my supervisor know I have a problem.
          }
      }
}

// Overloaded Virtual Functions

/*
INITIALIZE_NML

The INITIALIZE_NML function is ussually called only once from within the
constructor. It should not be called if a derived class will also call it.

*/
void RBF_NN_MODULE::INITIALIZE_NML()
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	setErrorLogChannel(new NML(nmlErrorFormat, "errlog", "rbf_nn", "rbf_module.nml"));
	setCmdChannel(new RCS_CMD_CHANNEL(rbf_nnFormat, "rbf_nn_cmd", "rbf_nn", "rbf_module.nml"));
	rbf_nn_status = new RBF_NN_STATUS();
	setStatChannel(new RCS_STAT_CHANNEL(rbf_nnFormat, "rbf_nn_sts", "rbf_nn", "rbf_module.nml"), rbf_nn_status);
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


}

/*
PRE_PROCESS

The PRE_PROCESS function is called every cycle after the command and
subordinates status have been read but before DECISION_PROCESS is called.
It is intended to be used for tasks such as sensory processing that should
be performed every cycle regardless of the current command or state.
*/
void RBF_NN_MODULE::PRE_PROCESS()
{
	// Pre-Processing Code
  // Read and/or calculate inputs to the RBF NN
  // Here we just assign them to zero

  // System inputs
  for(int ii = 0; ii < num_of_inputs; ii++)
    {
      sys_inputs[ii] = 0.0;
    }

  // Read and/or calculate the output errors 
  // Here we just assign them to zero
  for(int ii = 0; ii < num_of_outputs; ii++)
    {
      output_errors[ii] = 0.0;
    }

}

/*
DECISION_PROCESS

The DECISION_PROCESS function is called every cycle as long as there is a non-zero command.
It is expected to call a command function based on commandInData->type.
*/
void RBF_NN_MODULE::DECISION_PROCESS()
{
	switch(commandInData->type)
	{
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	case RBF_NN_INIT_TYPE:
		INIT((RBF_NN_INIT *)commandInData);
		break;

	case RBF_NN_HALT_TYPE:
		HALT((RBF_NN_HALT *)commandInData);
		break;

	case RBF_NN_RUN_TYPE:
		RUN((RBF_NN_RUN *)commandInData);
		break;

	case RBF_NN_SET_ADAPTATION_GAIN_TYPE:
		SET_ADAPTATION_GAIN((RBF_NN_SET_ADAPTATION_GAIN *)commandInData);
		break;

	case RBF_NN_SET_CENTERS_TYPE:
		SET_CENTERS((RBF_NN_SET_CENTERS *)commandInData);
		break;

	case RBF_NN_SET_OUT_PARAMS_TYPE:
		SET_OUT_PARAMS((RBF_NN_SET_OUT_PARAMS *)commandInData);
		break;

	case RBF_NN_SET_SPREADS_TYPE:
		SET_SPREADS((RBF_NN_SET_SPREADS *)commandInData);
		break;
		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	default:
		logError("The command %d is not recognized.",commandInData->type);
		break;
	}
}

/*
POST_PROCESS

The POST_PROCESS function is called every cycle after DECISION_PROCESS is called
but before the status and the subordinates commands  have been written.
It is intended to be used for tasks such as output filters that should
be performed every cycle regardless of the current command or state.
*/
void RBF_NN_MODULE::POST_PROCESS()
{
	// Post-Processing Code
  static int output_index = 0;

  // Report the current system inputs
  memcpy(rbf_nn_status->sys_inputs, sys_inputs, 
	 num_of_inputs*sizeof(double));

  // Report the current system outputs
  memcpy(rbf_nn_status->sys_outputs, sys_outputs, 
	 num_of_outputs*sizeof(double));

  // Report the current output errors
  memcpy(rbf_nn_status->output_errors, output_errors, 
	 num_of_outputs*sizeof(double));

  // Report the current adaptation gain used
  rbf_nn_status->adaptation_gain = rbf->get_eta();
}

// Command Functions

/*
INIT

Parameter(s):
RBF_NN_INIT *cmd_in -- NML Message sent from superior.

Most Modules will have an INIT command.
The INIT function is expected to initialize any variables that may be
in an uninitialized or unknown state, send INIT commands to its subordinates,
wait for the subordinates to be DONE and then inform its superior that it is done.
The state tables should use the STATE_MATCH macro so the diagnostics tool can 
highlight the current line in the state table.
*/
void RBF_NN_MODULE::INIT(RBF_NN_INIT *cmd_in)
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	if(STATE_MATCH(NEW_COMMAND))
	{
		// Send an INIT command to all subordinates.
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


		stateNext(S1);
		// Reinitialize variables here.

	}
	// Wait for all subordinates to report done.
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	else if(STATE_MATCH(S1,
		1))
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

	{
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
HALT

Parameter(s):
RBF_NN_HALT *cmd_in -- NML Message sent from superior.

Most Modules will have an HALT command.
The HALT function is expected to stop any motion or command execution,
send HALT commands to its subordinates,
wait for the subordinates to be DONE and then inform its superior that it is done.
The state tables should use the STATE_MATCH macro so the diagnostics tool can 
highlight the current line in the state table.
*/
void RBF_NN_MODULE::HALT(RBF_NN_HALT *cmd_in)
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	if(STATE_MATCH(NEW_COMMAND))
	{
		//Send a HALT command to all subordinates.
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

		stateNext(S1);
	}
	// Wait for all subordinates to report done.
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	else if(STATE_MATCH(S1,
		1))
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

	{
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
RUN

Parameter(s):
RBF_NN_RUN *cmd_in -- NML Message sent from superior.

*/
void RBF_NN_MODULE::RUN(RBF_NN_RUN *cmd_in)
{
	// Put state table for RBF_NN_RUN here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      // Calculate the system output
      rbf->control(sys_inputs, sys_outputs);

      // Update output parameters if necessary
      // Should read the error from PRE_PROCESS function
      //      rbf->update_out_params(output_error);

    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }

}

/*
SET_ADAPTATION_GAIN

Parameter(s):
RBF_NN_SET_ADAPTATION_GAIN *cmd_in -- NML Message sent from superior.

*/
void RBF_NN_MODULE::SET_ADAPTATION_GAIN(RBF_NN_SET_ADAPTATION_GAIN *cmd_in)
{
	// Put state table for RBF_NN_SET_ADAPTATION_GAIN here.
        if(STATE_MATCH(NEW_COMMAND))
        {
                status = RCS_EXEC;
                stateNext(S1);
        }
        else if(STATE_MATCH(S1))
        {
                rbf->set_eta(cmd_in->eta);

                status = RCS_DONE;
                stateNext(S2);
        }
        else if(STATE_MATCH(S2))
        {
                // Idle State
        }
}

/*
SET_CENTERS

Parameter(s):
RBF_NN_SET_CENTERS *cmd_in -- NML Message sent from superior.

*/
void RBF_NN_MODULE::SET_CENTERS(RBF_NN_SET_CENTERS *cmd_in)
{
	// Put state table for RBF_NN_SET_CENTERS here.
  int ii, jj;

  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      for (ii = 0; ii < num_of_neurons; ii++)
	{
	  for (jj = 0; jj < num_of_inputs; jj++)
	    {
	      temp[ii][jj] = cmd_in->centers[ii*num_of_inputs + jj];
	    }
	}
      rbf->set_centers(temp);
      
      status = RCS_DONE;
      stateNext(S2);
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

/*
SET_OUT_PARAMS

Parameter(s):
RBF_NN_SET_OUT_PARAMS *cmd_in -- NML Message sent from superior.

*/
void RBF_NN_MODULE::SET_OUT_PARAMS(RBF_NN_SET_OUT_PARAMS *cmd_in)
{
	// Put state table for RBF_NN_SET_OUT_PARAMS here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      // Set the all the TS coefficients for the specified rule 
      rbf->set_output_params(cmd_in->low_bound, cmd_in->high_bound);

      status = RCS_DONE;
      stateNext(S2);
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

/*
SET_SPREADS

Parameter(s):
RBF_NN_SET_SPREADS *cmd_in -- NML Message sent from superior.

*/
void RBF_NN_MODULE::SET_SPREADS(RBF_NN_SET_SPREADS *cmd_in)
{
	// Put state table for RBF_NN_SET_SPREADS here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      // Set the all the TS coefficients for the specified rule 
      rbf->set_spreads(cmd_in->spreads);

      status = RCS_DONE;
      stateNext(S2);
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

